attribute vec4 a_position;
attribute vec4 a_color;
attribute vec4 rt;
attribute vec3 i_offset;
varying vec4 v_color;
attribute vec2 a_texCoord0;
uniform mat4 u_projViewTrans;
varying vec2 v_texCoord0;

mat3 rotateY(float rad)
{
    float c = cos(rad);
    float s = sin(rad);
    return mat3(c, 0.0, -s, 0.0, 1.0, 0.0, s, 0.0, c);
}

mat3 rotateX(float rad)
{
    float c = cos(rad);
    float s = sin(rad);
    return mat3(1.0, 0.0, 0.0, 0.0, c, s, 0.0, -s, c);
}


mat3 rotateQ(vec3 axis, float rad)
{
    float hr = rad / 2.0;
    float s = sin(hr);
    vec4 q = vec4(axis * s, cos(hr));
    vec3 q2 = q.xyz * q.xyz;
    vec3 qq2 = q.xyz * q2;
    vec2 qx = q.xx * q2.yz;
    float qy = q.y * q2.z;
    vec3 qw = q.w * q2.xyz;
    return mat3(1.0 - (qq2.y + qq2.z), qx.x - qw.z, qx.y + qw.z, qx.x + qw.z, 1.0 - (qq2.x + qq2.z), qy - qw.x,
    qx.y - qw.y, qy + qw.x, 1.0 - (qq2.x + qq2.y));
}

mat4 rotationZ(float angle) {
    return mat4(cos(angle), -sin(angle), 0, 0, sin(angle), cos(angle), 0, 0, 0, 0, 1, 0, 0, 0, 0, 1);
}

mat4 rotationX(float angle)
{
    return mat4(1.0, 0.0, 0.0, 0.0,
    0.0, cos(angle), -sin(angle), 0.0,
    0.0, sin(angle), cos(angle), 0.0,
    0.0, 0.0, 0.0, 1.0);
}

mat4 rotationY(float angle)
{
    return mat4(cos(angle), 0.0, sin(angle), 0.0,
    0.0, 1.0, 0.0, 0.0,
    -sin(angle), 0.0, cos(angle), 0.0,
    0.0, 0.0, 0.0, 1.0);
}

void main () {
    v_color = a_color;
    v_texCoord0 = a_texCoord0;
    mat4 res = rotationX(rt.z) * rotationY(rt.w) * rotationZ(rt.y);
    vec4 view = a_position * res;
    vec4 trn = (view + vec4(i_offset, 0.0));
    gl_Position = u_projViewTrans * trn;
}